==> bosch-thermostat-client-0.8.2.tar.gz: bosch-thermostat-client-0.8.2/bosch_thermostat_client/circuits/circuit.py <==
"""Main circuit object."""
import logging
from bosch_thermostat_client.const import (
    ID,
    CURRENT_TEMP,
    OPERATION_MODE,
    REFS,
    HA_STATES,
    STATUS,
    TYPE,
    URI,
    RESULT,
    OFF,
    CIRCUIT_TYPES,
    ACTIVE_PROGRAM,
    MODE_TO_SETPOINT,
    HA_NAME,
    BOSCH_NAME,
    UNITS,
)
from bosch_thermostat_client.helper import BoschSingleEntity
from bosch_thermostat_client.exceptions import DeviceException

from .operation_mode import OperationModeHelper

_LOGGER = logging.getLogger(__name__)


class BasicCircuit(BoschSingleEntity):
    def __init__(self, connector, attr_id, db, _type, bus_type):
        """Basic circuit init."""
        name = attr_id.split("/").pop()
        self._db = db[CIRCUIT_TYPES[_type]]
        self._bus_type = bus_type
        super().__init__(name, connector, attr_id, _type)
        self._main_uri = f"/{CIRCUIT_TYPES[_type]}/{self.name}"
        self._operation_mode = {}
        for key, value in self._db[REFS].items():
            uri = f"{self._main_uri}/{value[ID]}"
            self._data[key] = {RESULT: {}, URI: uri, TYPE: value[TYPE]}

    @property
    def db_json(self):
        """Give simple json scheme of circuit."""
        return self._db

    async def update_requested_key(self, key):
        """Update info about Circuit asynchronously."""
        if key in self._data:
            try:
                result = await self._connector.get(self._data[key][URI])
                self.process_results(result, key)
                self._state = True
            except DeviceException:
                self._state = False

    @property
    def state(self):
        raise NotImplementedError

    async def initialize(self):
        """Check each uri if return json with values."""
        await self.update_requested_key(STATUS)


class Circuit(BasicCircuit):
    """Parent object for circuit of type HC or DHW."""

    def __init__(self, connector, attr_id, db, _type, bus_type, current_date=None):
        """Initialize circuit with get, put and id from gateway."""
        super().__init__(connector, attr_id, db, _type, bus_type)
        self._op_mode = OperationModeHelper(self.name, self._db.get(MODE_TO_SETPOINT))
        self._target_temp = 0

    @property
    def schedule(self):
        """Retrieve schedule of HC/DHW."""
        raise NotImplementedError

    @property
    def _hastates(self):
        """Get dictionary which converts Bosch states to HA States."""
        return self._db.get(HA_STATES, None)

    @property
    def setpoint(self):
        raise NotImplementedError

    async def set_service_call(self, uri, value):
        """WARNING! It doesn't check if value you send is good!."""
        _LOGGER.info(f"Sending service call {uri} with {value}")
        uri = f"{self._main_uri}/{uri}"
        val = await self._connector.put(uri, value)
        return val

    async def set_operation_mode(self, new_mode):
        """Set operation_mode of Heating Circuit."""
        if self._op_mode.current_mode == new_mode:
            _LOGGER.warning("Trying to set mode which is already set %s", new_mode)
            return None
        if new_mode in self._op_mode.available_modes:
            await self._connector.put(self._op_mode.uri, new_mode)
            self._op_mode.set_new_operation_mode(new_mode)
            return new_mode
        _LOGGER.warning(
            "You wanted to set %s, but it is not allowed %s",
            new_mode,
            self._op_mode.available_modes,
        )

    def _find_ha_mode(self, ha_mode):
        for v in self._hastates:
            if v[HA_NAME] == ha_mode:
                return v[BOSCH_NAME]

    @property
    def _temp_setpoint(self):
        return self._op_mode.temp_setpoint()

    async def set_ha_mode(self, ha_mode):
        """Helper to set operation mode."""
        old_setpoint = self._temp_setpoint
        old_mode = self._op_mode.current_mode
        bosch_mode = self._op_mode.find_in_available_modes(self._find_ha_mode(ha_mode))
        new_mode = await self.set_operation_mode(bosch_mode)
        different_mode = new_mode != old_mode
        try:
            if (
                different_mode
                and old_setpoint != self._temp_setpoint
                and self._op_mode.is_manual
            ):
                temp = self.get_value(self._temp_setpoint, 0)
                if temp == 0:
                    result = await self._connector.get(
                        self._data[self._temp_setpoint][URI]
                    )
                    self.process_results(result, self._temp_setpoint)
        except DeviceException as err:
            _LOGGER.debug(f"Can't update data for mode {new_mode}. Error: {err}")
            pass
        if different_mode:
            return 1
        return 0

    @property
    def current_temp(self):
        """Give current temperature of circuit."""
        _LOGGER.debug(
            "Current temp of %s is %s",
            self.name.upper(),
            self.get_property(CURRENT_TEMP),
        )
        temp = self.get_value(CURRENT_TEMP)
        if temp and temp > 0 and temp < 120:
            return temp

    @property
    def temp_units(self):
        """Return units of temperature."""
        return self.get_property(CURRENT_TEMP).get(UNITS)

    @property
    def ha_modes(self):
        """Retrieve HA modes."""
        return [
            v[HA_NAME]
            for v in self._hastates
            if any(x in self._op_mode.available_modes for x in v[BOSCH_NAME])
        ]

    @property
    def ha_mode(self):
        """Retrieve current mode in HA terminology."""
        for v in self._hastates:
            for x in v[BOSCH_NAME]:
                if x == self._op_mode.current_mode:
                    return v[HA_NAME]
        return OFF

    def get_activeswitchprogram(self, result=None):
        """
        Retrieve active program from thermostat.
        If ActiveSwitch program is not present
        then take first one from the list from result.
        """
        active_program = self.get_value(ACTIVE_PROGRAM)
        if active_program:
            return active_program
        try:
            return result["references"][0][ID].split("/")[-1]
        except IndexError:
            return None

    @property
    def target_temperature(self):
        """Get target temperature of Circtuit. Temporary or Room set point."""
        raise NotImplementedError

    @property
    def min_temp(self):
        """Retrieve minimum temperature."""
        raise NotImplementedError

    @property
    def max_temp(self):
        """Retrieve maximum temperature."""
        raise NotImplementedError

    async def set_temperature(self, temperature):
        """Set temperature of Circuit."""
        raise NotImplementedError

    async def update(self):
        """Update info about Circuit asynchronously."""
        _LOGGER.debug("Updating circuit %s", self.name)
        last_item = list(self._data.keys())[-1]
        for key, item in self._data.items():
            is_operation_type = item[TYPE] == OPERATION_MODE
            try:
                result = await self._connector.get(item[URI])
                self.process_results(result, key)
            except DeviceException:
                continue
            if item[TYPE] == ACTIVE_PROGRAM and self._schedule:
                active_program = self.get_activeswitchprogram(result)
                if active_program:
                    await self._schedule.update_schedule(active_program)
            if not self._op_mode.is_set and is_operation_type and result:
                self._op_mode.init_op_mode(
                    self.process_results(result, key, True), item[URI]
                )
            if key == last_item:
                self._state = True

==> bosch-thermostat-client-0.8.3.tar.gz: bosch-thermostat-client-0.8.3/bosch_thermostat_client/circuits/circuit.py <==
"""Main circuit object."""
import logging
from bosch_thermostat_client.const import (
    ID,
    CURRENT_TEMP,
    OPERATION_MODE,
    REFS,
    HA_STATES,
    STATUS,
    TYPE,
    URI,
    RESULT,
    OFF,
    CIRCUIT_TYPES,
    ACTIVE_PROGRAM,
    MODE_TO_SETPOINT,
    UNITS,
    BOSCH_NAME,
    HA_NAME
)
from bosch_thermostat_client.helper import BoschSingleEntity
from bosch_thermostat_client.exceptions import DeviceException

from .operation_mode import OperationModeHelper

_LOGGER = logging.getLogger(__name__)


class BasicCircuit(BoschSingleEntity):
    def __init__(self, connector, attr_id, db, _type, bus_type):
        """Basic circuit init."""
        name = attr_id.split("/").pop()
        self._db = db[CIRCUIT_TYPES[_type]]
        self._bus_type = bus_type
        super().__init__(name, connector, attr_id, _type)
        self._main_uri = f"/{CIRCUIT_TYPES[_type]}/{self.name}"
        self._operation_mode = {}
        for key, value in self._db[REFS].items():
            uri = f"{self._main_uri}/{value[ID]}"
            self._data[key] = {RESULT: {}, URI: uri, TYPE: value[TYPE]}

    @property
    def db_json(self):
        """Give simple json scheme of circuit."""
        return self._db

    async def update_requested_key(self, key):
        """Update info about Circuit asynchronously."""
        if key in self._data:
            try:
                result = await self._connector.get(self._data[key][URI])
                self.process_results(result, key)
                self._state = True
            except DeviceException:
                self._state = False

    @property
    def state(self):
        raise NotImplementedError

    async def initialize(self):
        """Check each uri if return json with values."""
        await self.update_requested_key(STATUS)


class Circuit(BasicCircuit):
    """Parent object for circuit of type HC or DHW."""

    def __init__(self, connector, attr_id, db, _type, bus_type, current_date=None):
        """Initialize circuit with get, put and id from gateway."""
        super().__init__(connector, attr_id, db, _type, bus_type)
        self._op_mode = OperationModeHelper(self.name, self._db.get(MODE_TO_SETPOINT))
        self._target_temp = 0

    @property
    def schedule(self):
        """Retrieve schedule of HC/DHW."""
        raise NotImplementedError

    @property
    def _hastates(self):
        """Get dictionary which converts Bosch states to HA States."""
        return self._db.get(HA_STATES, None)

    @property
    def setpoint(self):
        raise NotImplementedError

    async def set_service_call(self, uri, value):
        """WARNING! It doesn't check if value you send is good!."""
        _LOGGER.info(f"Sending service call {uri} with {value}")
        uri = f"{self._main_uri}/{uri}"
        val = await self._connector.put(uri, value)
        return val

    async def set_operation_mode(self, new_mode):
        """Set operation_mode of Heating Circuit."""
        if self._op_mode.current_mode == new_mode:
            _LOGGER.warning("Trying to set mode which is already set %s", new_mode)
            return None
        if new_mode in self._op_mode.available_modes:
            await self._connector.put(self._op_mode.uri, new_mode)
            self._op_mode.set_new_operation_mode(new_mode)
            return new_mode
        _LOGGER.warning(
            "You wanted to set %s, but it is not allowed %s",
            new_mode,
            self._op_mode.available_modes,
        )

    def _find_ha_mode(self, ha_mode):
        for v in self._hastates:
            if v[HA_NAME] == ha_mode:
                return v[BOSCH_NAME]

    @property
    def _temp_setpoint(self):
        return self._op_mode.temp_setpoint()

    async def set_ha_mode(self, ha_mode):
        """Helper to set operation mode."""
        old_setpoint = self._temp_setpoint
        old_mode = self._op_mode.current_mode
        bosch_mode = self._op_mode.find_in_available_modes(self._find_ha_mode(ha_mode))
        new_mode = await self.set_operation_mode(bosch_mode)
        different_mode = new_mode != old_mode
        try:
            if (
                different_mode
                and old_setpoint != self._temp_setpoint
                and self._op_mode.is_manual
            ):
                temp = self.get_value(self._temp_setpoint, 0)
                if temp == 0:
                    result = await self._connector.get(
                        self._data[self._temp_setpoint][URI]
                    )
                    self.process_results(result, self._temp_setpoint)
        except DeviceException as err:
            _LOGGER.debug(f"Can't update data for mode {new_mode}. Error: {err}")
            pass
        if different_mode:
            return 1
        return 0

    @property
    def current_temp(self):
        """Give current temperature of circuit."""
        _LOGGER.debug(
            "Current temp of %s is %s",
            self.name.upper(),
            self.get_property(CURRENT_TEMP),
        )
        temp = self.get_value(CURRENT_TEMP)
        if temp:
            temp = float(temp)
            if temp > 0 and temp < 120:
                return temp

    @property
    def temp_units(self):
        """Return units of temperature."""
        return self.get_property(CURRENT_TEMP).get(UNITS)

    @property
    def ha_modes(self):
        """Retrieve HA modes."""
        raise NotImplementedError

    @property
    def ha_mode(self):
        """Retrieve current mode in HA terminology."""
        for v in self._hastates:
            for x in v[BOSCH_NAME]:
                if x == self._op_mode.current_mode:
                    return v[HA_NAME]
        return OFF

    def get_activeswitchprogram(self, result=None):
        """
        Retrieve active program from thermostat.
        If ActiveSwitch program is not present
        then take first one from the list from result.
        """
        active_program = self.get_value(ACTIVE_PROGRAM)
        if active_program:
            return active_program
        try:
            return result["references"][0][ID].split("/")[-1]
        except IndexError:
            return None

    @property
    def target_temperature(self):
        """Get target temperature of Circtuit. Temporary or Room set point."""
        raise NotImplementedError

    @property
    def min_temp(self):
        """Retrieve minimum temperature."""
        raise NotImplementedError

    @property
    def max_temp(self):
        """Retrieve maximum temperature."""
        raise NotImplementedError

    async def set_temperature(self, temperature):
        """Set temperature of Circuit."""
        raise NotImplementedError

    async def update(self):
        """Update info about Circuit asynchronously."""
        _LOGGER.debug("Updating circuit %s", self.name)
        last_item = list(self._data.keys())[-1]
        for key, item in self._data.items():
            is_operation_type = item[TYPE] == OPERATION_MODE
            try:
                result = await self._connector.get(item[URI])
                self.process_results(result, key)
            except DeviceException:
                continue
            if item[TYPE] == ACTIVE_PROGRAM and self._schedule:
                active_program = self.get_activeswitchprogram(result)
                if active_program:
                    await self._schedule.update_schedule(active_program)
            if not self._op_mode.is_set and is_operation_type and result:
                self._op_mode.init_op_mode(
                    self.process_results(result, key, True), item[URI]
                )
            if key == last_item:
                self._state = True
